"""Tests for file IO"""

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import io as stdlib_io
import os.path
import unittest
import pytest
import stat
import sys

from ..fileio import atomic_writing

from ipython_genutils.tempdir import TemporaryDirectory

umask = 0

def test_atomic_writing():
    class CustomExc(Exception): pass

    with TemporaryDirectory() as td:
        f1 = os.path.join(td, 'penguin')
        with stdlib_io.open(f1, 'w') as f:
            f.write(u'Before')
        
        if os.name != 'nt':
            os.chmod(f1, 0o701)
            orig_mode = stat.S_IMODE(os.stat(f1).st_mode)

        f2 = os.path.join(td, 'flamingo')
        try:
            os.symlink(f1, f2)
            have_symlink = True
        except (AttributeError, NotImplementedError, OSError):
            # AttributeError: Python doesn't support it
            # NotImplementedError: The system doesn't support it
            # OSError: The user lacks the privilege (Windows)
            have_symlink = False

        with pytest.raises(CustomExc):
            with atomic_writing(f1) as f:
                f.write(u'Failing write')
                raise CustomExc

        # Because of the exception, the file should not have been modified
        with stdlib_io.open(f1, 'r') as f:
            assert f.read() == u'Before'

        with atomic_writing(f1) as f:
            f.write(u'Overwritten')

        with stdlib_io.open(f1, 'r') as f:
            assert f.read() == u'Overwritten'

        if os.name != 'nt':
            mode = stat.S_IMODE(os.stat(f1).st_mode)
            assert mode == orig_mode

        if have_symlink:
            # Check that writing over a file preserves a symlink
            with atomic_writing(f2) as f:
                f.write(u'written from symlink')
            
            with stdlib_io.open(f1, 'r') as f:
                assert f.read() == u'written from symlink'

class TestWithSetUmask(unittest.TestCase):
    def setUp(self):
        # save umask
        global umask
        umask = os.umask(0)
        os.umask(umask)
    
    def tearDown(self):
        # restore umask
        os.umask(umask)

    @pytest.mark.skipif(sys.platform == "win32", reason="do not run on windows")
    def test_atomic_writing_umask(self):
        with TemporaryDirectory() as td:
            os.umask(0o022)
            f1 = os.path.join(td, '1')
            with atomic_writing(f1) as f:
                f.write(u'1')
            mode = stat.S_IMODE(os.stat(f1).st_mode)
            assert mode == 0o644
    
            os.umask(0o057)
            f2 = os.path.join(td, '2')
            with atomic_writing(f2) as f:
                f.write(u'2')
            mode = stat.S_IMODE(os.stat(f2).st_mode)
            assert mode == 0o620


def test_atomic_writing_newlines():
    with TemporaryDirectory() as td:
        path = os.path.join(td, 'testfile')
        
        lf = u'a\nb\nc\n'
        plat = lf.replace(u'\n', os.linesep)
        crlf = lf.replace(u'\n', u'\r\n')
        
        # test default
        with stdlib_io.open(path, 'w') as f:
            f.write(lf)
        with stdlib_io.open(path, 'r', newline='') as f:
            read = f.read()
        assert read == plat
        
        # test newline=LF
        with stdlib_io.open(path, 'w', newline='\n') as f:
            f.write(lf)
        with stdlib_io.open(path, 'r', newline='') as f:
            read = f.read()
        assert read == lf
        
        # test newline=CRLF
        with atomic_writing(path, newline='\r\n') as f:
            f.write(lf)
        with stdlib_io.open(path, 'r', newline='') as f:
            read = f.read()
        assert read == crlf
        
        # test newline=no convert
        text = u'crlf\r\ncr\rlf\n'
        with atomic_writing(path, newline='') as f:
            f.write(text)
        with stdlib_io.open(path, 'r', newline='') as f:
            read = f.read()
        assert read == text



"""Test the bundlers API."""

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import io
from os.path import join as pjoin

from notebook.tests.launchnotebook import NotebookTestBase
from nbformat import write
from nbformat.v4 import (
    new_notebook, new_markdown_cell, new_code_cell, new_output,
)

from unittest.mock import patch

    
def bundle(handler, model):
    """Bundler test stub. Echo the notebook path."""
    handler.finish(model['path'])

class BundleAPITest(NotebookTestBase):
    """Test the bundlers web service API"""
    @classmethod
    def setup_class(cls):
        """Make a test notebook. Borrowed from nbconvert test. Assumes the class
        teardown will clean it up in the end."""
        super(BundleAPITest, cls).setup_class()
        nbdir = cls.notebook_dir

        nb = new_notebook()

        nb.cells.append(new_markdown_cell(u'Created by test'))
        cc1 = new_code_cell(source=u'print(2*6)')
        cc1.outputs.append(new_output(output_type="stream", text=u'12'))
        nb.cells.append(cc1)
        
        with io.open(pjoin(nbdir, 'testnb.ipynb'), 'w',
                     encoding='utf-8') as f:
            write(nb, f, version=4)

    def test_missing_bundler_arg(self):
        """Should respond with 400 error about missing bundler arg"""
        resp = self.request('GET', 'bundle/fake.ipynb')
        self.assertEqual(resp.status_code, 400)
        self.assertIn('Missing argument bundler', resp.text)

    def test_notebook_not_found(self):
        """Should respond with 404 error about missing notebook"""
        resp = self.request('GET', 'bundle/fake.ipynb',
            params={'bundler': 'fake_bundler'})
        self.assertEqual(resp.status_code, 404)
        self.assertIn('Not Found', resp.text)

    def test_bundler_not_enabled(self):
        """Should respond with 400 error about disabled bundler"""
        resp = self.request('GET', 'bundle/testnb.ipynb',
            params={'bundler': 'fake_bundler'})
        self.assertEqual(resp.status_code, 400)
        self.assertIn('Bundler fake_bundler not enabled', resp.text)

    def test_bundler_import_error(self):
        """Should respond with 500 error about failure to load bundler module"""
        with patch('notebook.bundler.handlers.BundlerHandler.get_bundler') as mock:
            mock.return_value = {'module_name': 'fake_module'}
            resp = self.request('GET', 'bundle/testnb.ipynb',
                params={'bundler': 'fake_bundler'})
            mock.assert_called_with('fake_bundler')
        self.assertEqual(resp.status_code, 500)
        self.assertIn('Could not import bundler fake_bundler', resp.text)
        
    def test_bundler_invoke(self):
        """Should respond with 200 and output from test bundler stub"""
        with patch('notebook.bundler.handlers.BundlerHandler.get_bundler') as mock:
            mock.return_value = {'module_name': 'notebook.bundler.tests.test_bundler_api'}
            resp = self.request('GET', 'bundle/testnb.ipynb',
                params={'bundler': 'stub_bundler'})
            mock.assert_called_with('stub_bundler')
        self.assertEqual(resp.status_code, 200)
        self.assertIn('testnb.ipynb', resp.text)